@@ -1,6 +1,6 @@
 /* export.c
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
- *               2004 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004,
+ *               2005 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -16,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -34,6 +35,7 @@
 #include "util.h"
 #include "main.h"
 #include "i18n.h"
+#include "trustdb.h"
 
 static int do_export( STRLIST users, int secret, unsigned int options );
 static int do_export_stream( IOBUF out, STRLIST users, int secret,
@@ -48,11 +50,19 @@ parse_export_options(char *str,unsigned int *options,int noisy)
       {"export-local-sigs",EXPORT_LOCAL_SIGS,NULL},
       {"export-attributes",EXPORT_ATTRIBUTES,NULL},
       {"export-sensitive-revkeys",EXPORT_SENSITIVE_REVKEYS,NULL},
-      {"export-minimal",EXPORT_MINIMAL,NULL},
+      {"export-minimal",EXPORT_MINIMAL|EXPORT_CLEAN_SIGS|EXPORT_CLEAN_UIDS,NULL},
+      {"export-clean",EXPORT_CLEAN_SIGS|EXPORT_CLEAN_UIDS,NULL},
+      {"export-clean-sigs",EXPORT_CLEAN_SIGS,NULL},
+      {"export-clean-uids",EXPORT_CLEAN_UIDS,NULL},
+
+      {"export-reset-subkey-passwd", EXPORT_RESET_SUBKEY_PASSWD, NULL},
+
       /* Aliases for backward compatibility */
       {"include-local-sigs",EXPORT_LOCAL_SIGS,NULL},
       {"include-attributes",EXPORT_ATTRIBUTES,NULL},
       {"include-sensitive-revkeys",EXPORT_SENSITIVE_REVKEYS,NULL},
+      /* dummy */
+      {"export-unusable-sigs",0,NULL},
       {NULL,0,NULL}
       /* add tags for include revoked and disabled? */
     };
@@ -220,13 +230,17 @@ do_export_stream( IOBUF out, STRLIST users, int secret,
 			 keystr(sk_keyid));
 		continue;
 	      }
-
-	    if(options&EXPORT_MINIMAL)
-	      keyid_from_sk(sk,keyid);
 	  }
-	else if((options&EXPORT_MINIMAL)
-		&& (node=find_kbnode(keyblock,PKT_PUBLIC_KEY)))
-	  keyid_from_pk(node->pkt->pkt.public_key,keyid);
+	else
+	  {
+	    /* It's a public key export */
+	    if((options&EXPORT_MINIMAL)
+	       && (node=find_kbnode(keyblock,PKT_PUBLIC_KEY)))
+	      keyid_from_pk(node->pkt->pkt.public_key,keyid);
+
+	    if(options&EXPORT_CLEAN_UIDS)
+	      clean_uids_from_key(keyblock,opt.verbose);
+	  }
 
 	/* and write it */
 	for( kbctx=NULL; (node = walk_kbnode( keyblock, &kbctx, 0 )); ) {
@@ -239,10 +253,11 @@ do_export_stream( IOBUF out, STRLIST users, int secret,
 		  continue;
 	      }
 
-	    /* don't export any comment packets but those in the
-	     * secret keyring */
-	    if( !secret && node->pkt->pkttype == PKT_COMMENT )
-		continue;
+	    /* We used to use comment packets, but not any longer.  In
+	       case we still have comments on a key, strip them here
+	       before we call build_packet(). */
+	    if( node->pkt->pkttype == PKT_COMMENT )
+	      continue;
 
             /* make sure that ring_trust packets never get exported */
             if (node->pkt->pkttype == PKT_RING_TRUST)
@@ -312,9 +327,16 @@ do_export_stream( IOBUF out, STRLIST users, int secret,
 		  continue;
 	      }
 
-	    if( node->pkt->pkttype == PKT_SIGNATURE )
+	    if(node->pkt->pkttype==PKT_USER_ID)
 	      {
-		/* If we have minimal-export turned on, do not include
+		/* Run clean_sigs_from_uid against each uid if
+		   export-clean-sigs is on. */
+		if(options&EXPORT_CLEAN_SIGS)
+		  clean_sigs_from_uid(keyblock,node,opt.verbose);
+	      }
+	    else if(node->pkt->pkttype==PKT_SIGNATURE)
+	      {
+		/* If we have export-minimal turned on, do not include
 		   any signature that isn't a selfsig.  Note that this
 		   only applies to uid sigs (0x10, 0x11, 0x12, and
 		   0x13).  A designated revocation is not stripped. */
@@ -324,7 +346,8 @@ do_export_stream( IOBUF out, STRLIST users, int secret,
 		       || node->pkt->pkt.signature->keyid[1]!=keyid[1]))
 		  continue;
 
-		/* do not export packets which are marked as not exportable */
+		/* do not export packets which are marked as not
+		   exportable */
 		if(!(options&EXPORT_LOCAL_SIGS)
 		   && !node->pkt->pkt.signature->flags.exportable)
 		  continue; /* not exportable */
@@ -362,7 +385,7 @@ do_export_stream( IOBUF out, STRLIST users, int secret,
 
 	    if( secret == 2 && node->pkt->pkttype == PKT_SECRET_KEY )
 	      {
-		/* we don't want to export the secret parts of the
+		/* We don't want to export the secret parts of the
 		 * primary key, this is done by using GNU protection mode 1001
 		 */
 		int save_mode = node->pkt->pkt.secret_key->protect.s2k.mode;
@@ -370,6 +393,53 @@ do_export_stream( IOBUF out, STRLIST users, int secret,
 		rc = build_packet( out, node->pkt );
 		node->pkt->pkt.secret_key->protect.s2k.mode = save_mode;
 	      }
+	    else if (secret == 2 && node->pkt->pkttype == PKT_SECRET_SUBKEY
+                     && (opt.export_options&EXPORT_RESET_SUBKEY_PASSWD))
+              {
+                /* If the subkey is protected reset the passphrase to
+                   export an unprotected subkey.  This feature is
+                   useful in cases of a subkey copied to an unattended
+                   machine where a passphrase is not required. */
+                PKT_secret_key *sk_save, *sk;
+
+                sk_save = node->pkt->pkt.secret_key;
+                sk = copy_secret_key (NULL, sk_save);
+                node->pkt->pkt.secret_key = sk;
+
+                log_info ("about to export an unprotected subkey\n");
+                switch (is_secret_key_protected (sk))
+                  {
+                  case -1:
+                    rc = G10ERR_PUBKEY_ALGO;
+                    break;
+                  case 0:
+                    break;
+                  default:
+                    if (sk->protect.s2k.mode == 1001)
+                      ; /* No secret parts. */
+                    else if( sk->protect.s2k.mode == 1002 ) 
+                      ; /* Card key stub. */
+                    else 
+                      {
+                        rc = check_secret_key( sk, 0 );
+                      }
+                    break;
+                  }
+                if (rc)
+                  {
+                    node->pkt->pkt.secret_key = sk_save;
+                    free_secret_key (sk);
+                    /* FIXME: Make translatable after releasing 1.4.2 */
+                    log_error ("failed to unprotect the subkey: %s\n",
+                               g10_errstr (rc));
+                    goto leave;
+                  }
+
+		rc = build_packet (out, node->pkt);
+
+                node->pkt->pkt.secret_key = sk_save;
+                free_secret_key (sk);
+              }
 	    else
 	      {
 		/* Warn the user if the secret key or any of the secret
