@@ -16,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -46,7 +47,7 @@
 #endif
 #include <errno.h>
 #include <ctype.h>
-#ifdef HAVE_READLINE_READLINE_H
+#ifdef HAVE_LIBREADLINE
 #include <readline/readline.h>
 #include <readline/history.h>
 #endif
@@ -162,9 +163,10 @@ init_ttyfp(void)
                   tty_get_ttyname (), strerror(errno) );
 	exit(2);
     }
-#if defined(HAVE_READLINE_READLINE_H) && defined(HAVE_LIBREADLINE)
+#ifdef HAVE_LIBREADLINE
     rl_catch_signals = 0;
     rl_instream = rl_outstream = ttyfp;
+    rl_inhibit_completion = 1;
 #endif
 #endif
 #ifdef HAVE_TCGETATTR
@@ -173,6 +175,25 @@ init_ttyfp(void)
     initialized = 1;
 }
 
+#ifdef HAVE_LIBREADLINE
+void
+tty_enable_completion(rl_completion_func_t *completer)
+{
+  if( !initialized )
+    init_ttyfp();
+  rl_attempted_completion_function=completer;
+  rl_inhibit_completion=0;
+}
+
+void
+tty_disable_completion(void)
+{
+  if( !initialized )
+    init_ttyfp();
+  rl_inhibit_completion=1;
+}
+#endif /*HAVE_LIBREADLINE*/
+
 int
 tty_batchmode( int onoff )
 {
@@ -376,7 +397,6 @@ do_get( const char *prompt, int hidden )
 	init_ttyfp();
 
     last_prompt_len = 0;
-    tty_printf( "%s", prompt );
     buf = m_alloc(n=50);
     i = 0;
 
@@ -384,6 +404,8 @@ do_get( const char *prompt, int hidden )
     if( hidden )
 	SetConsoleMode(con.in, HID_INPMODE );
 
+    tty_printf( "%s", prompt );
+
     for(;;) {
 	DWORD nread;
 
@@ -415,6 +437,7 @@ do_get( const char *prompt, int hidden )
 	SetConsoleMode(con.in, DEF_INPMODE );
 
 #elif defined(__riscos__)
+    tty_printf( "%s", prompt );
     do {
         c = riscos_getchar();
         if (c == 0xa || c == 0xd) { /* Return || Enter */
@@ -469,6 +492,8 @@ do_get( const char *prompt, int hidden )
 #endif
     }
 
+    tty_printf( "%s", prompt );
+
     /* fixme: How can we avoid that the \n is echoed w/o disabling
      * canonical mode - w/o this kill_prompt can't work */
     while( read(fileno(ttyfp), cbuf, 1) == 1 && *cbuf != '\n' ) {
@@ -512,7 +537,7 @@ do_get( const char *prompt, int hidden )
 char *
 tty_get( const char *prompt )
 {
-#if defined(HAVE_READLINE_READLINE_H) && defined(HAVE_LIBREADLINE)
+#ifdef HAVE_LIBREADLINE
     if (!batchmode && !no_terminal) {
       char *line;
       char *buf;
@@ -544,7 +569,7 @@ tty_get( const char *prompt )
       return buf;
     }
     else
-#endif /* HAVE_READLINE_READLINE_H && HAVE_LIBREADLINE */
+#endif /* HAVE_LIBREADLINE */
       return do_get( prompt, 0 );
 }
 
